﻿using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System;

namespace PEDataWin.Controls
{

    // Description:		A rounded group box frame object with various painting options
    // Date Created:	August 2009
    // Author:			Kevin Carbis
    // Author Email:	kacarbis@gmail.com
    // Websites:		www.staffinder.net/carbisoft
    // Version			1.0 - 2/11/2010 initial release
    // Version:			1.1 - 4/28/2011
    //							CaptionOrientation - This property allows you move the caption area so it docks on the Top, Left, Right, or Bottom.
    //							CaptionPadding - This allows you to add some spacing so you can fine tune the positioning of the Caption Text.&nbsp; Be sure to check these values when changing the Orientation to Left or Right.
    //							GroupBoxStyle - This property allows you to toggle between the standard and extended drawing techniques of the groupbox. I ran into a case where the stock groupbox had flicker problems but mine didn't.
    //							Added two new gradient options called HorizontalGlow and VerticalGlow. This draws the gradient from the center outwards in the direction specified.
    //							Fixed Tabbing issue by inheriting from GroupBox instead of UserControl.
    //							Fixed a couple minor bugs here and there.

    [Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", typeof(IDesigner))]
    public class RoundedGroupBox : GroupBox
    {
        private System.Drawing.Color _BorderColor = Color.FromArgb(141, 178, 227);
        private float _BorderWidth = 1;
        private System.Drawing.Color _BackgroundColor = Color.White;
        private System.Drawing.Color _BackgroundColorGradient = Color.FromArgb(227, 235, 246);
        private GroupBoxGradientMode _BackgroundGradientMode = GroupBoxGradientMode.Vertical;
        private string _Caption = "";
        private Color _CaptionBackColor = Color.FromArgb(194, 217, 240);
        private int _CaptionHeight = 25;
        private bool _CaptionVisible = true;
        private GroupBoxCaptionAlignment _CaptionAlignment = GroupBoxCaptionAlignment.Center;
        private int _CornerRadius = 5;
        private GroupBoxCorners _Corners = GroupBoxCorners.All;
        private int _DropShadowThickness = 3;
        private bool _DropShadowVisible = true;
        private System.Drawing.Color _ShadowColor = Color.FromArgb(50, Color.Black);
        private GroupBoxCaptionOrientation _CaptionOrientation = GroupBoxCaptionOrientation.Top;
        private Padding _CaptionPadding;

        private GroupBoxStyles _GroupBoxStyle = GroupBoxStyles.Extended;
        
        private bool _AllowCollaps = false;
        private Rectangle ArrowRec = new Rectangle(0, 0, 20, 20);
        //private bool _Collapsed = false;
        //private int _ActualHeight;
        //private int _ActualWidth;

        #region Enums

        public enum GroupBoxGradientMode
        {
            /// <summary>Specifies no gradient mode.</summary>
            None = 4,

            /// <summary>Specifies a gradient from upper right to lower left.</summary>
            BackwardDiagonal = LinearGradientMode.BackwardDiagonal,

            /// <summary>Specifies a gradient from upper left to lower right.</summary>
            ForwardDiagonal = LinearGradientMode.ForwardDiagonal,

            /// <summary>Specifies a gradient from left to right.</summary>
            Horizontal = LinearGradientMode.Horizontal,

            /// <summary>Specifies a gradient from top to bottom.</summary>
            Vertical = LinearGradientMode.Vertical,

            VerticalGlow = 5,
            HorizontalGlow = 6
        }

        public enum GroupBoxCorners
        {
            None = 0,
            NorthWest = 2,
            NorthEast = 4,
            SouthEast = 8,
            SouthWest = 16,
            All = NorthWest | NorthEast | SouthEast | SouthWest,
            North = NorthWest | NorthEast,
            South = SouthEast | SouthWest,
            East = NorthEast | SouthEast,
            West = NorthWest | SouthWest
        }
        public enum GroupBoxCaptionAlignment
        {
            Left = StringAlignment.Near,
            Right = StringAlignment.Far,
            Center = StringAlignment.Center
        }
        public enum GroupBoxCaptionOrientation
        {
            Top,
            Left,
            Right,
            Bottom
        }
        public enum GroupBoxStyles
        {
            Standard,
            Extended
        }

        public enum ArrowDirection
        {
            /// <summary>
            /// The arrow points up
            /// </summary>
            Up,

            /// <summary>
            /// The arrow points down
            /// </summary>
            Down,

            /// <summary>
            /// The arrow points right
            /// </summary>
            Right,

            /// <summary>
            /// The arrow points left
            /// </summary>
            Left
        }
        #endregion

        #region ctor

        public RoundedGroupBox()
        {
            SetStyle(ControlStyles.ResizeRedraw, true);
            SetStyle(ControlStyles.Selectable, false);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            SetStyle(ControlStyles.ContainerControl, true);

            this.TabStop = false;
            _CaptionPadding = new Padding(20, 20, 20, 20);
            base.MinimumSize = new Size(_CornerRadius * 2, _CornerRadius * 2);
        }

        #endregion

        #region Properties
        /// <summary>Gets or sets the radius of the corners of the control.</summary>
        [Category("Appearance"), Description("This feature will round the corners of the control.")]
        [DefaultValue(5)]
        public int CornerRadius
        {
            get { return _CornerRadius; }
            set
            {
                if (value > 35)
                {
                    _CornerRadius = 35;
                }
                else
                {
                    if (value < 1)
                    {
                        _CornerRadius = 1;
                    }
                    else
                    {
                        _CornerRadius = value;
                    }
                }
                Invalidate();
            }
        }
        /// <summary>Turns on or off the control shadowing.</summary>
        [Category("Appearance"), Description("This feature will turn on control shadowing.")]
        public bool DropShadowVisible
        {
            get { return _DropShadowVisible; }
            set
            {
                _DropShadowVisible = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the color of the control's border.</summary>
        [Category("Appearance"), Description("This feature will allow you to change the color of the control's border.")]
        public System.Drawing.Color BorderColor
        {
            get { return _BorderColor; }
            set
            {
                _BorderColor = value;
                Invalidate();
            }
        }
        /// <summary>Gets or Sets the control's border size.</summary>
        [Category("Appearance"), Description("This feature will allow you to set the control's border size.")]
        public float BorderWidth
        {
            get { return _BorderWidth; }
            set
            {
                if (value > 10)
                {
                    _BorderWidth = 10;
                }
                else
                {
                    if (value < 1)
                    {
                        _BorderWidth = 1;
                    }
                    else
                    {
                        _BorderWidth = value;
                    }
                }
                Invalidate();
            }
        }
        /// <summary>Gets or sets the size of the shadow border thickness.</summary>
        [Category("Appearance"), Description("This feature will change the size of the shadow border.")]
        public int DropShadowThickness
        {
            get { return _DropShadowThickness; }
            set
            {
                if (value > 10)
                {
                    _DropShadowThickness = 10;
                }
                else
                {
                    if (value < 1)
                    {
                        _DropShadowThickness = 1;
                    }
                    else
                    {
                        _DropShadowThickness = value;
                    }
                }
                Invalidate();
            }
        }
        /// <summary>Gets or sets the background color to use. This color can also be used in combination with BackgroundColorGradient for a gradient paint.</summary>
        [Category("Appearance"), Description("This feature will change the group control color. This color can also be used in combination with BackgroundColorGradient for a gradient paint.")]
        public System.Drawing.Color BackgroundColor
        {
            get { return _BackgroundColor; }
            set
            {
                _BackgroundColor = value;
                Invalidate();
            }
        }
        /// <summary>Specifies the second color when using the gradient mode.</summary>
        [Category("Appearance"), Description("This feature can be used in combination with BackgroundColor to create a gradient background.")]
        public System.Drawing.Color BackgroundColorGradient
        {
            get { return _BackgroundColorGradient; }
            set
            {
                _BackgroundColorGradient = value;
                Invalidate();
            }
        }
        // <summary>Here we hide the borderstyle property from the properties page and intellisence.</summary>
        //<Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        //Public Overloads Property BorderStyle() As BorderStyle
        //   Get
        //      Return MyBase.BorderStyle
        //   End Get
        //   Set(ByVal value As BorderStyle)
        //      MyBase.BorderStyle = value
        //   End Set
        //End Property
        /// <summary>This property secifies the type of gradient to use when painting the background.</summary>
        [Category("Appearance"), Description("This feature turns on background gradient painting.")]
        public GroupBoxGradientMode BackgroundGradientMode
        {
            get { return _BackgroundGradientMode; }
            set
            {
                _BackgroundGradientMode = value;
                Invalidate();
            }
        }
        /// <summary>Specifies teh background color for the caption bar.</summary>
        [Category("Appearance"), Description("This feature will paint the group title background to the specified color if PaintGroupBox is set to true.")]
        public System.Drawing.Color CaptionBackColor
        {
            get { return _CaptionBackColor; }
            set
            {
                _CaptionBackColor = value;
                Invalidate();
            }
        }
        /// <summary>The text to diplay in the caption bar</summary>
        public string Caption
        {
            get { return _Caption; }
            set
            {
                _Caption = value;
                Invalidate();
            }
        }
        /// <summary>Specifies which corners to apply rounding for</summary>
        [DefaultValue(GroupBoxCorners.All)]
        public GroupBoxCorners Corners
        {
            get { return _Corners; }
            set
            {
                _Corners = value;
                Invalidate();
            }
        }
        /// <summary>Turns on or off the Caption Bar</summary>
        [DefaultValue(true)]
        public bool CaptionVisible
        {
            get { return _CaptionVisible; }
            set
            {
                _CaptionVisible = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the height of the caption bar</summary>
        [DefaultValue(30)]
        public int CaptionHeight
        {
            get { return _CaptionHeight; }
            set
            {
                _CaptionHeight = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the alignment of the caption text</summary>
        [Category("Appearance"), Description("Gets or sets the alignment of the caption text.")]
        [DefaultValue(GroupBoxCaptionAlignment.Center)]
        public GroupBoxCaptionAlignment CaptionAlignment
        {
            get { return _CaptionAlignment; }
            set
            {
                _CaptionAlignment = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the orientation of the Caption</summary>
        [Category("Appearance"), Description("Gets or sets the orientation of the Caption.")]
        [DefaultValue(GroupBoxCaptionOrientation.Top)]
        public GroupBoxCaptionOrientation CaptionOrientation
        {
            get { return _CaptionOrientation; }
            set
            {
                _CaptionOrientation = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the Padding for the Caption</summary>
        [Category("Appearance"), DefaultValue("20,20,20,20")]
        public Padding CaptionPadding
        {
            get { return _CaptionPadding; }
            set
            {
                _CaptionPadding = value;
                Invalidate();
            }
        }
        /// <summary>Gets or sets the Style for the GroupBox.  Standard duplicates the stock groupbox.  Extended allows the customization ability</summary>
        [Category("Appearance"), Description("Gets or sets the Style for the GroupBox.  Standard duplicates the stock groupbox.  Extended allows the customization ability")]
        [DefaultValue(GroupBoxStyles.Extended)]
        public GroupBoxStyles GroupBoxStyle
        {
            get { return _GroupBoxStyle; }
            set
            {
                _GroupBoxStyle = value;
                Invalidate();
            }
        }
        /// <summary>
        /// Gets or Sets if the Groupbox can be collapsed
        /// </summary>
        [DefaultValue(false)]
        public bool AllowCollaps
        {
            get { return _AllowCollaps; }
            set { _AllowCollaps = value; Invalidate(); }
        }
        #endregion

        #region Painting
        protected override void OnPaint(PaintEventArgs e)
        {
            if (_GroupBoxStyle == GroupBoxStyles.Standard)
            {
                base.OnPaint(e);
                return;
            }

            //we must set the smoothing mode to anti alias or high quality(They are the same) in order to get the
            //nice rounded corders on our control
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            //first lets set the rectangles we will be drawing in.  If the drop shadow is visible then we need to 
            //reduce the size of the main rectangle and create a second one that is offset by the shadow thickness.
            Rectangle shadowRec = default(Rectangle);
            Rectangle frameRec = default(Rectangle);
            if (DropShadowVisible)
            {
                shadowRec = Rectangle.FromLTRB(DropShadowThickness, DropShadowThickness, this.ClientRectangle.Right - 1, this.ClientRectangle.Bottom - 1);
                frameRec = Rectangle.FromLTRB(0, 0, this.ClientRectangle.Right - DropShadowThickness - 2, this.ClientRectangle.Bottom - DropShadowThickness - 2);
            }
            else
            {
                frameRec = Rectangle.FromLTRB(this.ClientRectangle.Left, this.ClientRectangle.Top, this.ClientRectangle.Right - 1, this.ClientRectangle.Bottom - 1);
            }

            if (frameRec.Height <= 0 | frameRec.Width <= 0)
                return;

            //draw the drop shadow using the shadow path rectangle
            if (DropShadowVisible)
            {
                using (GraphicsPath path = RoundRectangle(shadowRec, _CornerRadius, _Corners))
                {
                    using (SolidBrush b = new SolidBrush(_ShadowColor))
                    {
                        e.Graphics.FillPath(b, path);
                    }
                }
            }

            //draw the rest of the control in the main frame path
            using (GraphicsPath path = RoundRectangle(frameRec, _CornerRadius, _Corners))
            {
                //paint the background inside the frame
                if (BackgroundGradientMode == GroupBoxGradientMode.None)
                {
                    //solid background
                    using (SolidBrush b = new SolidBrush(BackgroundColor))
                    {
                        e.Graphics.FillPath(b, path);
                    }
                }
                else
                {
                    //gradient
                    if (BackgroundGradientMode == GroupBoxGradientMode.VerticalGlow)
                    {
                        using (LinearGradientBrush b = new LinearGradientBrush(new Point(0, frameRec.Top + (frameRec.Height / 2)), new Point(0, frameRec.Bottom - 1), BackgroundColorGradient, BackgroundColor))
                        {

                            b.WrapMode = WrapMode.TileFlipXY;
                            e.Graphics.FillPath(b, path);
                        }
                    }
                    else if (BackgroundGradientMode == GroupBoxGradientMode.HorizontalGlow)
                    {
                        using (LinearGradientBrush b = new LinearGradientBrush(new Point(frameRec.Left + (frameRec.Width / 2), 0), new Point(frameRec.Right - 1, 0), BackgroundColorGradient, BackgroundColor))
                        {

                            b.WrapMode = WrapMode.TileFlipXY;
                            e.Graphics.FillPath(b, path);
                        }
                    }
                    else
                    {
                        using (LinearGradientBrush br = new LinearGradientBrush(frameRec, BackgroundColor, BackgroundColorGradient, (LinearGradientMode)BackgroundGradientMode))
                        {
                            e.Graphics.FillPath(br, path);
                        }
                    }
                }

                //the caption area needs a new path rectangle but must have the same corner radius for the top corners.
                if (CaptionVisible)
                {
                    //determine our caption path based on the orientation
                    StringFormat sf = new StringFormat();
                    Rectangle textRec = default(Rectangle);
                    Rectangle captionRec = default(Rectangle);
                    GroupBoxCorners captionCorners = default(GroupBoxCorners);
                    switch (CaptionOrientation)
                    {
                        case GroupBoxCaptionOrientation.Top:
                            captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Top, frameRec.Right, frameRec.Top + CaptionHeight);
                            captionCorners = _Corners & GroupBoxCorners.North;
                            textRec = captionRec;
                            textRec.Width = captionRec.Width - (CaptionPadding.Left + CaptionPadding.Right);
                            textRec.X = CaptionPadding.Left;
                            break;
                        case GroupBoxCaptionOrientation.Left:
                            captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Top, frameRec.Left + CaptionHeight, frameRec.Bottom);
                            captionCorners = _Corners & GroupBoxCorners.West;
                            sf.FormatFlags = StringFormatFlags.DirectionVertical;
                            textRec = captionRec;
                            textRec.Height = captionRec.Height - (CaptionPadding.Top + CaptionPadding.Bottom);
                            textRec.Y += CaptionPadding.Bottom;
                            break;
                        case GroupBoxCaptionOrientation.Right:
                            captionRec = Rectangle.FromLTRB(frameRec.Right - CaptionHeight, frameRec.Top, frameRec.Right, frameRec.Bottom);
                            captionCorners = _Corners & GroupBoxCorners.East;
                            sf.FormatFlags = StringFormatFlags.DirectionVertical;
                            textRec = captionRec;
                            textRec.Height = captionRec.Height - (CaptionPadding.Top + CaptionPadding.Bottom);
                            textRec.Y = CaptionPadding.Top;
                            break;
                        case GroupBoxCaptionOrientation.Bottom:
                            captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Bottom - CaptionHeight, frameRec.Right, frameRec.Bottom);
                            captionCorners = _Corners & GroupBoxCorners.South;
                            textRec = captionRec;
                            textRec.Width = captionRec.Width - (CaptionPadding.Left + CaptionPadding.Right);
                            textRec.X = CaptionPadding.Left;
                            break;
                    }

                    //draw the caption background
                    using (GraphicsPath capPath = RoundRectangle(captionRec, _CornerRadius, captionCorners))
                    {
                        using (SolidBrush b = new SolidBrush(CaptionBackColor))
                        {
                            e.Graphics.FillPath(b, capPath);
                        }
                    }

                    //draw the border
                    using (Pen p = new Pen(BorderColor, BorderWidth))
                    {
                        e.Graphics.DrawPath(p, path);
                    }

                    if (!string.IsNullOrEmpty(Caption))
                    {
                        //draw the caption text
                        SizeF tSize = e.Graphics.MeasureString(Caption, this.Font);
                        sf.Alignment = (StringAlignment)CaptionAlignment;
                        sf.LineAlignment = StringAlignment.Center;
                        if (CaptionOrientation == GroupBoxCaptionOrientation.Left)
                        {
                            //we want to draw from bottom to top so flip the text 180
                            e.Graphics.TranslateTransform(textRec.Width, textRec.Height);
                            //e.Graphics.TranslateTransform(textRec.Left + (textRec.Width \ 2), textRec.Top + (textRec.Height \ 2), Drawing2D.MatrixOrder.Append)
                            e.Graphics.RotateTransform(180);
                        }
                        using (SolidBrush b = new SolidBrush(ForeColor))
                        {
                            e.Graphics.DrawString(Caption, this.Font, b, textRec, sf);
                        }
                        e.Graphics.ResetTransform();
                        //Using p As Pen = New Pen(BorderColor, BorderWidth)
                        //   e.Graphics.DrawRectangle(p, textRec)
                        //End Using
                    }
                    if (_AllowCollaps)
                    {
                        int maxHeight = captionRec.Height > 20 ? 20 : captionRec.Height;
                        int maxWidth = captionRec.Width > 20 ? 20 : captionRec.Width;

                        ArrowDirection arrow = ArrowDirection.Down;
                        if (CaptionOrientation == GroupBoxCaptionOrientation.Left)
                        {
                            //use the top area for the rec and draw a right arrow
                            ArrowRec.Width = maxWidth;
                            ArrowRec.Height = maxWidth;
                            ArrowRec.Y = CornerRadius;
                            ArrowRec.X = Convert.ToInt32(captionRec.Width - ArrowRec.Width / 2);
                            arrow = ArrowDirection.Right;
                        }
                        else if (CaptionOrientation == GroupBoxCaptionOrientation.Right)
                        {
                            ArrowRec = Rectangle.FromLTRB(captionRec.Left, captionRec.Bottom - 20, captionRec.Right, captionRec.Bottom);
                            arrow = ArrowDirection.Left;
                        }
                        else if (CaptionOrientation == GroupBoxCaptionOrientation.Bottom)
                        {
                            ArrowRec = Rectangle.FromLTRB(captionRec.Left, captionRec.Top, captionRec.Right, captionRec.Right - 20);
                            arrow = ArrowDirection.Up;
                        }
                        else if (CaptionOrientation == GroupBoxCaptionOrientation.Top)
                        {
                            ArrowRec = new Rectangle(captionRec.Right - CornerRadius - maxWidth,
                                                           captionRec.Top + Convert.ToInt32(ArrowRec.Height / 2),
                                                           maxHeight,
                                                           maxHeight);
                            arrow = ArrowDirection.Down;
                        }
                        using (SolidBrush b = new SolidBrush(Color.White))
                        {
                            e.Graphics.FillEllipse(b, ArrowRec);
                        }
                        ArrowRec.Inflate(-4, -5);
                        //ArrowRec.Offset(1, 1);
                        DrawArrow(e.Graphics, ArrowRec, _CaptionBackColor, arrow);
                    }
                }
                e.Graphics.ResetTransform();
                //draw the border
                using (Pen p = new Pen(BorderColor, BorderWidth))
                {
                    e.Graphics.DrawPath(p, path);
                }
            }
        }
        private GraphicsPath RoundRectangle(Rectangle r, int radius, GroupBoxCorners corners)
        {
            GraphicsPath path = new GraphicsPath();
            if (r.Width <= 0 | r.Height <= 0)
                return path;

            int d = radius * 2;

            int nw = ((corners & GroupBoxCorners.NorthWest) == GroupBoxCorners.NorthWest ? d : 0);
            int ne = ((corners & GroupBoxCorners.NorthEast) == GroupBoxCorners.NorthEast ? d : 0);
            int se = ((corners & GroupBoxCorners.SouthEast) == GroupBoxCorners.SouthEast ? d : 0);
            int sw = ((corners & GroupBoxCorners.SouthWest) == GroupBoxCorners.SouthWest ? d : 0);

            path.AddLine(r.Left + nw, r.Top, r.Right - ne, r.Top);

            if (ne > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Right - ne, r.Top, r.Right, r.Top + ne), -90, 90);
            }

            path.AddLine(r.Right, r.Top + ne, r.Right, r.Bottom - se);

            if (se > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Right - se, r.Bottom - se, r.Right, r.Bottom), 0, 90);
            }

            path.AddLine(r.Right - se, r.Bottom, r.Left + sw, r.Bottom);

            if (sw > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Left, r.Bottom - sw, r.Left + sw, r.Bottom), 90, 90);
            }

            path.AddLine(r.Left, r.Bottom - sw, r.Left, r.Top + nw);

            if (nw > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Left, r.Top, r.Left + nw, r.Top + nw), 180, 90);
            }

            path.CloseFigure();
            return path;
        }
        private void DrawArrow(Graphics g, Rectangle b, Color c, ArrowDirection d)
        {
            GraphicsPath path = new GraphicsPath();
            Rectangle bounds = b;

            if (b.Width % 2 != 0 && (d == ArrowDirection.Up))
                bounds = new Rectangle(new Point(b.Left - 1, b.Top - 1), new Size(b.Width + 1, b.Height + 1));

            if (d == ArrowDirection.Up)
            {
                path.AddLine(bounds.Left, bounds.Bottom, bounds.Right, bounds.Bottom);
                path.AddLine(bounds.Right, bounds.Bottom, bounds.Left + bounds.Width / 2, bounds.Top);
            }
            else if (d == ArrowDirection.Down)
            {
                path.AddLine(bounds.Left, bounds.Top, bounds.Right, bounds.Top);
                path.AddLine(bounds.Right, bounds.Top, bounds.Left + bounds.Width / 2, bounds.Bottom);
            }
            else if (d == ArrowDirection.Left)
            {
                path.AddLine(bounds.Left, bounds.Top, bounds.Right, bounds.Top + bounds.Height / 2);
                path.AddLine(bounds.Right, bounds.Top + bounds.Height / 2, bounds.Left, bounds.Bottom);
            }
            else
            {
                path.AddLine(bounds.Right, bounds.Top, bounds.Left, bounds.Top + bounds.Height / 2);
                path.AddLine(bounds.Left, bounds.Top + bounds.Height / 2, bounds.Right, bounds.Bottom);
            }

            path.CloseFigure();

            using (SolidBrush bb = new SolidBrush(c))
            {
                SmoothingMode sm = g.SmoothingMode;
                g.SmoothingMode = SmoothingMode.None;
                g.FillPath(bb, path);
                g.SmoothingMode = sm;
            }

            path.Dispose();
        }
        /// <summary>
        /// Draws the pair of arrows that make a shadded arrow, centered on the specified bounds
        /// </summary>
        /// <param name="g"></param>
        /// <param name="b"></param>
        /// <param name="d"></param>
        /// <param name="enabled"></param>
        //public void DrawArrowShaded(Graphics g, Rectangle b, ArrowDirection d, bool enabled, Color color)
        //{
        //    Size arrSize = new Size(15, 15);

        //    if (d == ArrowDirection.Left || d == ArrowDirection.Right)
        //    {
        //        //Invert size
        //        arrSize = new Size(arrSize.Height, arrSize.Width);
        //    }

        //    Point arrowP = new Point(
        //         b.Left + (b.Width - arrSize.Width) / 2,
        //         b.Top + (b.Height - arrSize.Height) / 2
        //         );

        //    Rectangle bounds = new Rectangle(arrowP, arrSize);
        //    Rectangle boundsLight = bounds; boundsLight.Offset(0, 1);

        //    Color lt = ColorTable.ArrowLight;
        //    Color dk = ColorTable.Arrow;

        //    if (!enabled)
        //    {
        //        lt = Color.Transparent;
        //        dk = ColorTable.ArrowDisabled;
        //    }

        //    DrawArrow(g, boundsLight, lt, d);
        //    DrawArrow(g, bounds, dk, d);
        //}
    }
        #endregion

}
