﻿' Description:		A rounded group box frame object with various painting options
' Date Created:	August 2009
' Author:			Kevin Carbis
' Author Email:	kacarbis@gmail.com / kevin@staffinder.net
' Websites:			www.staffinder.net/carbisoft
' Credits:			I took the RoundRectangle function from www.menendezpoo.com.
'						His was more compact and simpler than my version
' License:			If you use this control in a comertial application you need to link back to my site.
' Version			1.0 - 2/11/2010 initial release
' Version:			1.1 - 4/28/2011
'							CaptionOrientation - This property allows you move the caption area so it docks on the Top, Left, Right, or Bottom.
'							CaptionPadding - This allows you to add some spacing so you can fine tune the positioning of the Caption Text.&nbsp; Be sure to check these values when changing the Orientation to Left or Right.
'							GroupBoxStyle - This property allows you to toggle between the standard and extended drawing techniques of the groupbox. I ran into a case where the stock groupbox had flicker problems but mine didn't.
'							Added two new gradient options called HorizontalGlow and VerticalGlow. This draws the gradient from the center outwards in the direction specified.
'							Fixed Tabbing issue by inheriting from GroupBox instead of UserControl.
'							Fixed a couple minor bugs here and there.

Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.ComponentModel.Design

<Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", GetType(IDesigner))> _
Public Class RoundedGroupBox
	Inherits GroupBox

	Private _BorderColor As System.Drawing.Color = Color.FromArgb(141, 178, 227)
	Private _BorderWidth As Single = 1
	Private _BackgroundColor As System.Drawing.Color = Color.White
	Private _BackgroundColorGradient As System.Drawing.Color = Color.FromArgb(227, 235, 246)
	Private _BackgroundGradientMode As GroupBoxGradientMode = GroupBoxGradientMode.Vertical
	Private _Caption As String = ""
	Private _CaptionBackColor As Color = Color.FromArgb(194, 217, 240)
	Private _CaptionHeight As Integer = 25
	Private _CaptionVisible As Boolean = True
	Private _CaptionAlignment As GroupBoxCaptionAlignment = GroupBoxCaptionAlignment.Center
	Private _CornerRadius As Integer = 5
	Private _Corners As GroupBoxCorners = GroupBoxCorners.All
	Private _DropShadowThickness As Integer = 3
	Private _DropShadowVisible As Boolean = True
	Private _ShadowColor As System.Drawing.Color = Color.FromArgb(50, Color.Black)
	Private _CaptionOrientation As GroupBoxCaptionOrientation = GroupBoxCaptionOrientation.Top
	Private _CaptionPadding As Padding
	Private _GroupBoxStyle As GroupBoxStyles = GroupBoxStyles.Extended

	Public Enum GroupBoxGradientMode
		''' <summary>Specifies no gradient mode.</summary>
		None = 4

		''' <summary>Specifies a gradient from upper right to lower left.</summary>
		BackwardDiagonal = LinearGradientMode.BackwardDiagonal

		''' <summary>Specifies a gradient from upper left to lower right.</summary>
		ForwardDiagonal = LinearGradientMode.ForwardDiagonal

		''' <summary>Specifies a gradient from left to right.</summary>
		Horizontal = LinearGradientMode.Horizontal

		''' <summary>Specifies a gradient from top to bottom.</summary>
		Vertical = LinearGradientMode.Vertical

		VerticalGlow = 5
		HorizontalGlow = 6
	End Enum
	<Flags()> _
	Public Enum GroupBoxCorners
		None = 0
		NorthWest = 2
		NorthEast = 4
		SouthEast = 8
		SouthWest = 16
		All = NorthWest Or NorthEast Or SouthEast Or SouthWest
		North = NorthWest Or NorthEast
		South = SouthEast Or SouthWest
		East = NorthEast Or SouthEast
		West = NorthWest Or SouthWest
	End Enum
	Public Enum GroupBoxCaptionAlignment
		Left = StringAlignment.Near
		Right = StringAlignment.Far
		Center = StringAlignment.Center
	End Enum
	Public Enum GroupBoxCaptionOrientation
		Top
		Left
		Right
		Bottom
	End Enum
	Public Enum GroupBoxStyles
		Standard
		Extended
	End Enum

	Public Sub New()
		SetStyle(ControlStyles.ResizeRedraw, True)
		SetStyle(ControlStyles.Selectable, False)
		SetStyle(ControlStyles.AllPaintingInWmPaint, True)
		SetStyle(ControlStyles.OptimizedDoubleBuffer, True)
		SetStyle(ControlStyles.SupportsTransparentBackColor, True)
		SetStyle(ControlStyles.ContainerControl, True)

		Me.TabStop = False
		_CaptionPadding = New Padding(20, 20, 20, 20)
		MyBase.MinimumSize = New Size(_CornerRadius * 2, _CornerRadius * 2)
	End Sub

	''' <summary>Gets or sets the radius of the corners of the control.</summary>
	<Category("Appearance"), Description("This feature will round the corners of the control.")> _
	Public Property CornerRadius() As Integer
		Get
			Return _CornerRadius
		End Get
		Set(ByVal value As Integer)
			If value > 35 Then
				_CornerRadius = 35
			Else
				If value < 1 Then
					_CornerRadius = 1
				Else
					_CornerRadius = value
				End If
			End If
			Invalidate()
		End Set
	End Property
	''' <summary>Turns on or off the control shadowing.</summary>
	<Category("Appearance"), Description("This feature will turn on control shadowing.")> _
	Public Property DropShadowVisible() As Boolean
		Get
			Return _DropShadowVisible
		End Get
		Set(ByVal value As Boolean)
			_DropShadowVisible = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the color of the control's border.</summary>
	<Category("Appearance"), Description("This feature will allow you to change the color of the control's border.")> _
	Public Property BorderColor() As System.Drawing.Color
		Get
			Return _BorderColor
		End Get
		Set(ByVal value As System.Drawing.Color)
			_BorderColor = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or Sets the control's border size.</summary>
	<Category("Appearance"), Description("This feature will allow you to set the control's border size.")> _
	Public Property BorderWidth() As Single
		Get
			Return _BorderWidth
		End Get
		Set(ByVal value As Single)
			If value > 10 Then
				_BorderWidth = 10
			Else
				If value < 1 Then
					_BorderWidth = 1
				Else
					_BorderWidth = value
				End If
			End If
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the size of the shadow border thickness.</summary>
	<Category("Appearance"), Description("This feature will change the size of the shadow border.")> _
	Public Property DropShadowThickness() As Integer
		Get
			Return _DropShadowThickness
		End Get
		Set(ByVal value As Integer)
			If value > 10 Then
				_DropShadowThickness = 10
			Else
				If value < 1 Then
					_DropShadowThickness = 1
				Else
					_DropShadowThickness = value
				End If
			End If
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the background color to use. This color can also be used in combination with BackgroundColorGradient for a gradient paint.</summary>
	<Category("Appearance"), Description("This feature will change the group control color. This color can also be used in combination with BackgroundColorGradient for a gradient paint.")> _
	Public Property BackgroundColor() As System.Drawing.Color
		Get
			Return _BackgroundColor
		End Get
		Set(ByVal value As System.Drawing.Color)
			_BackgroundColor = value
			Invalidate()
		End Set
	End Property
	''' <summary>Specifies the second color when using the gradient mode.</summary>
	<Category("Appearance"), Description("This feature can be used in combination with BackgroundColor to create a gradient background.")> _
	Public Property BackgroundColorGradient() As System.Drawing.Color
		Get
			Return _BackgroundColorGradient
		End Get
		Set(ByVal value As System.Drawing.Color)
			_BackgroundColorGradient = value
			Invalidate()
		End Set
	End Property
	' <summary>Here we hide the borderstyle property from the properties page and intellisence.</summary>
	'<Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
	'Public Overloads Property BorderStyle() As BorderStyle
	'   Get
	'      Return MyBase.BorderStyle
	'   End Get
	'   Set(ByVal value As BorderStyle)
	'      MyBase.BorderStyle = value
	'   End Set
	'End Property
	''' <summary>This property secifies the type of gradient to use when painting the background.</summary>
	<Category("Appearance"), Description("This feature turns on background gradient painting.")> _
	Public Property BackgroundGradientMode() As GroupBoxGradientMode
		Get
			Return _BackgroundGradientMode
		End Get
		Set(ByVal value As GroupBoxGradientMode)
			_BackgroundGradientMode = value
			Invalidate()
		End Set
	End Property
	''' <summary>Specifies teh background color for the caption bar.</summary>
	<Category("Appearance"), Description("This feature will paint the group title background to the specified color if PaintGroupBox is set to true.")> _
	Public Property CaptionBackColor() As System.Drawing.Color
		Get
			Return _CaptionBackColor
		End Get
		Set(ByVal value As System.Drawing.Color)
			_CaptionBackColor = value
			Invalidate()
		End Set
	End Property
	''' <summary>The text to diplay in the caption bar</summary>
	Public Property Caption() As String
		Get
			Return _Caption
		End Get
		Set(ByVal value As String)
			_Caption = value
			Invalidate()
		End Set
	End Property
	''' <summary>Specifies which corners to apply rounding for</summary>
	Public Property Corners() As GroupBoxCorners
		Get
			Return _Corners
		End Get
		Set(ByVal value As GroupBoxCorners)
			_Corners = value
			Invalidate()
		End Set
	End Property
	''' <summary>Turns on or off the Caption Bar</summary>
	Public Property CaptionVisible() As Boolean
		Get
			Return _CaptionVisible
		End Get
		Set(ByVal value As Boolean)
			_CaptionVisible = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the height of the caption bar</summary>
	Public Property CaptionHeight() As Integer
		Get
			Return _CaptionHeight
		End Get
		Set(ByVal value As Integer)
			_CaptionHeight = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the alignment of the caption text</summary>
	<Category("Appearance"), Description("Gets or sets the alignment of the caption text.")> _
	<DefaultValue(GroupBoxCaptionAlignment.Center)> _
	Public Property CaptionAlignment() As GroupBoxCaptionAlignment
		Get
			Return _CaptionAlignment
		End Get
		Set(ByVal value As GroupBoxCaptionAlignment)
			_CaptionAlignment = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the orientation of the Caption</summary>
	<Category("Appearance"), Description("Gets or sets the orientation of the Caption.")> _
	<DefaultValue(GroupBoxCaptionOrientation.Top)> _
	Public Property CaptionOrientation() As GroupBoxCaptionOrientation
		Get
			Return _CaptionOrientation
		End Get
		Set(ByVal value As GroupBoxCaptionOrientation)
			_CaptionOrientation = value
			Invalidate()
		End Set
	End Property
	''' <summary>Gets or sets the Padding for the Caption</summary>
	<Category("Appearance"), DefaultValue("20,20,20,20")> _
	Public Property CaptionPadding() As Padding
		Get
			Return _CaptionPadding
		End Get
		Set(ByVal value As Padding)
			_CaptionPadding = value
			Invalidate()
		End Set
	End Property

	''' <summary>Gets or sets the Style for the GroupBox.  Standard duplicates the stock groupbox.  Extended allows the customization ability</summary>
	<Category("Appearance"), Description("Gets or sets the Style for the GroupBox.  Standard duplicates the stock groupbox.  Extended allows the customization ability")> _
	<DefaultValue(GroupBoxStyles.Extended)> _
	Public Property GroupBoxStyle() As GroupBoxStyles
		Get
			Return _GroupBoxStyle
		End Get
		Set(ByVal value As GroupBoxStyles)
			_GroupBoxStyle = value
			Invalidate()
		End Set
	End Property

	Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
		If _GroupBoxStyle = GroupBoxStyles.Standard Then
			MyBase.OnPaint(e)
			Return
		End If

		'we must set the smoothing mode to anti alias or high quality(They are the same) in order to get the
		'nice rounded corders on our control
		e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

		'first lets set the rectangles we will be drawing in.  If the drop shadow is visible then we need to 
		'reduce the size of the main rectangle and create a second one that is offset by the shadow thickness.
		Dim shadowRec As Rectangle
		Dim frameRec As Rectangle
		If DropShadowVisible Then
			shadowRec = Rectangle.FromLTRB(DropShadowThickness, DropShadowThickness, Me.ClientRectangle.Right - 1, Me.ClientRectangle.Bottom - 1)
			frameRec = Rectangle.FromLTRB(0, 0, Me.ClientRectangle.Right - DropShadowThickness - 2, Me.ClientRectangle.Bottom - DropShadowThickness - 2)
		Else
			frameRec = Rectangle.FromLTRB(Me.ClientRectangle.Left, Me.ClientRectangle.Top, Me.ClientRectangle.Right - 1, Me.ClientRectangle.Bottom - 1)
		End If

		If frameRec.Height <= 0 Or frameRec.Width <= 0 Then Return

		'draw the drop shadow using the shadow path rectangle
		If DropShadowVisible Then
			Using path As GraphicsPath = RoundRectangle(shadowRec, _CornerRadius, _Corners)
				Using b As SolidBrush = New SolidBrush(_ShadowColor)
					e.Graphics.FillPath(b, path)
				End Using
			End Using
		End If

		'draw the rest of the control in the main frame path
		Using path As GraphicsPath = RoundRectangle(frameRec, _CornerRadius, _Corners)
			'paint the background inside the frame
			If BackgroundGradientMode = GroupBoxGradientMode.None Then
				'solid background
				Using b As SolidBrush = New SolidBrush(BackgroundColor)
					e.Graphics.FillPath(b, path)
				End Using
			Else
				'gradient
				If BackgroundGradientMode = GroupBoxGradientMode.VerticalGlow Then
					Using b As LinearGradientBrush = New LinearGradientBrush( _
						 New Point(0, frameRec.Top + (frameRec.Height \ 2)), _
						 New Point(0, frameRec.Bottom - 1), BackgroundColorGradient, BackgroundColor)

						b.WrapMode = WrapMode.TileFlipXY
						e.Graphics.FillPath(b, path)
					End Using
				ElseIf BackgroundGradientMode = GroupBoxGradientMode.HorizontalGlow Then
					Using b As LinearGradientBrush = New LinearGradientBrush( _
						 New Point(frameRec.Left + (frameRec.Width \ 2), 0), _
						 New Point(frameRec.Right - 1, 0), BackgroundColorGradient, BackgroundColor)

						b.WrapMode = WrapMode.TileFlipXY
						e.Graphics.FillPath(b, path)
					End Using
				Else
					Using br As LinearGradientBrush = New LinearGradientBrush(frameRec, BackgroundColor, BackgroundColorGradient, CType(BackgroundGradientMode, LinearGradientMode))
						e.Graphics.FillPath(br, path)
					End Using
				End If
			End If

			'the caption area needs a new path rectangle but must have the same corner radius for the top corners.
			If CaptionVisible Then
				'determine our caption path based on the orientation
				Dim sf As New StringFormat
				Dim textRec As Rectangle
				Dim captionRec As Rectangle
				Dim captionCorners As GroupBoxCorners
				Select Case CaptionOrientation
					Case GroupBoxCaptionOrientation.Top
						captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Top, frameRec.Right, frameRec.Top + CaptionHeight)
						captionCorners = _Corners And GroupBoxCorners.North
						textRec = captionRec
						textRec.Width = captionRec.Width - (CaptionPadding.Left + CaptionPadding.Right)
						textRec.X = CaptionPadding.Left
					Case GroupBoxCaptionOrientation.Left
						captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Top, frameRec.Left + CaptionHeight, frameRec.Bottom)
						captionCorners = _Corners And GroupBoxCorners.West
						sf.FormatFlags = StringFormatFlags.DirectionVertical
						textRec = captionRec
						textRec.Height = captionRec.Height - (CaptionPadding.Top + CaptionPadding.Bottom)
						textRec.Y += CaptionPadding.Bottom
					Case GroupBoxCaptionOrientation.Right
						captionRec = Rectangle.FromLTRB(frameRec.Right - CaptionHeight, frameRec.Top, frameRec.Right, frameRec.Bottom)
						captionCorners = _Corners And GroupBoxCorners.East
						sf.FormatFlags = StringFormatFlags.DirectionVertical
						textRec = captionRec
						textRec.Height = captionRec.Height - (CaptionPadding.Top + CaptionPadding.Bottom)
						textRec.Y = CaptionPadding.Top
					Case GroupBoxCaptionOrientation.Bottom
						captionRec = Rectangle.FromLTRB(frameRec.Left, frameRec.Bottom - CaptionHeight, frameRec.Right, frameRec.Bottom)
						captionCorners = _Corners And GroupBoxCorners.South
						textRec = captionRec
						textRec.Width = captionRec.Width - (CaptionPadding.Left + CaptionPadding.Right)
						textRec.X = CaptionPadding.Left
				End Select

				'draw the caption background
				Using capPath As GraphicsPath = RoundRectangle(captionRec, _CornerRadius, captionCorners)
					Using b As SolidBrush = New SolidBrush(CaptionBackColor)
						e.Graphics.FillPath(b, capPath)
					End Using
				End Using

				'draw the border
				Using p As Pen = New Pen(BorderColor, BorderWidth)
					e.Graphics.DrawPath(p, path)
				End Using

				If Caption <> "" Then
					'draw the caption text
					Dim tSize As SizeF = e.Graphics.MeasureString(Caption, Me.Font)
					sf.Alignment = CaptionAlignment
					sf.LineAlignment = StringAlignment.Center
					If CaptionOrientation = GroupBoxCaptionOrientation.Left Then
						'we want to draw from bottom to top so flip the text 180
						e.Graphics.TranslateTransform(textRec.Width, textRec.Height)
						'e.Graphics.TranslateTransform(textRec.Left + (textRec.Width \ 2), textRec.Top + (textRec.Height \ 2), Drawing2D.MatrixOrder.Append)
						e.Graphics.RotateTransform(180)
					End If
					Using b As SolidBrush = New SolidBrush(ForeColor)
                  e.Graphics.DrawString(Caption, Me.Font, b, RectangleF.op_Implicit(textRec), sf)
					End Using
					e.Graphics.ResetTransform()
					'Using p As Pen = New Pen(BorderColor, BorderWidth)
					'   e.Graphics.DrawRectangle(p, textRec)
					'End Using
				End If
			End If
			e.Graphics.ResetTransform()
			'draw the border
			Using p As Pen = New Pen(BorderColor, BorderWidth)
				e.Graphics.DrawPath(p, path)
			End Using
		End Using
	End Sub
	Private Function RoundRectangle(ByVal r As Rectangle, ByVal radius As Integer, ByVal corners As GroupBoxCorners) As GraphicsPath
		Dim path As GraphicsPath = New GraphicsPath()
		If r.Width <= 0 Or r.Height <= 0 Then Return path

		Dim d As Integer = radius * 2

		Dim nw As Integer = IIf((corners And GroupBoxCorners.NorthWest) = GroupBoxCorners.NorthWest, d, 0)
		Dim ne As Integer = IIf((corners And GroupBoxCorners.NorthEast) = GroupBoxCorners.NorthEast, d, 0)
		Dim se As Integer = IIf((corners And GroupBoxCorners.SouthEast) = GroupBoxCorners.SouthEast, d, 0)
		Dim sw As Integer = IIf((corners And GroupBoxCorners.SouthWest) = GroupBoxCorners.SouthWest, d, 0)

		path.AddLine(r.Left + nw, r.Top, r.Right - ne, r.Top)

		If ne > 0 Then
			path.AddArc(Rectangle.FromLTRB(r.Right - ne, r.Top, r.Right, r.Top + ne), -90, 90)
		End If

		path.AddLine(r.Right, r.Top + ne, r.Right, r.Bottom - se)

		If se > 0 Then
			path.AddArc(Rectangle.FromLTRB(r.Right - se, r.Bottom - se, r.Right, r.Bottom), 0, 90)
		End If

		path.AddLine(r.Right - se, r.Bottom, r.Left + sw, r.Bottom)

		If sw > 0 Then
			path.AddArc(Rectangle.FromLTRB(r.Left, r.Bottom - sw, r.Left + sw, r.Bottom), 90, 90)
		End If

		path.AddLine(r.Left, r.Bottom - sw, r.Left, r.Top + nw)

		If nw > 0 Then
			path.AddArc(Rectangle.FromLTRB(r.Left, r.Top, r.Left + nw, r.Top + nw), 180, 90)
		End If

		path.CloseFigure()
		Return path
	End Function

End Class
